//
//  QBRTCConfigModule.m
//  quickblox_sdk
//
//  Created by Injoit on 14.01.2021.
//  Copyright © 2021 Injoit LTD. All rights reserved.
//

#import "QBRTCConfigModule.h"
#import <QuickbloxWebRTC/QuickbloxWebRTC.h>

@implementation QBRTCConfigModule

- (instancetype)init {
    self = [super init];
    if (self) {
        QBRTCConfig.answerTimeInterval = 60.0;
        QBRTCConfig.dialingTimeInterval = 5.0;
    }
    return self;
}

- (void)setAnswerTimeInterval:(NSNumber *)interval
                     resolver:(QBResolveBlock)resolve
                     rejecter:(QBRejectBlock)reject {
    NSTimeInterval minimumInterval =  10.0;
    
    if ([NSError reject:reject
           checkerClass:NSNumber.class
                 object:interval
              objectKey:@"interval"]) {
        return;
    }
    
    if (interval.doubleValue < minimumInterval) {
        [NSError reject:reject message:@"Value should be equal to or greater than 10"];
        return;
    }
    
    QBRTCConfig.answerTimeInterval = interval.doubleValue;
    resolve(nil);
}

- (void)getAnswerTimeInterval:(QBResolveBlock)resolve
                     rejecter:(QBRejectBlock)reject {
    NSNumber *result = @((NSInteger)QBRTCConfig.answerTimeInterval);
    resolve(result);
}

- (void)setDialingTimeInterval:(NSNumber *)interval
                      resolver:(QBResolveBlock)resolve
                      rejecter:(QBRejectBlock)reject {
    if ([NSError reject:reject
           checkerClass:NSNumber.class
                 object:interval
              objectKey:@"interval"]) {
        return;
    }
    
    if (interval.doubleValue < 3.0) {
        [NSError reject:reject message:@"Value should be equal to or greater than 3"];
        return;
    }
    
    QBRTCConfig.dialingTimeInterval = interval.doubleValue;
    resolve(nil);
}

- (void)getDialingTimeInterval:(QBResolveBlock)resolve
                      rejecter:(QBRejectBlock)reject {
    NSNumber *result = @((NSInteger)QBRTCConfig.dialingTimeInterval);
    resolve(result);
}

@end
